<?php
require_once __DIR__ . '/../../../init.php';
require_once __DIR__ . '/../../../includes/gatewayfunctions.php';
require_once __DIR__ . '/../../../includes/invoicefunctions.php';

use Illuminate\Database\Capsule\Manager as Capsule;
use WHMCS\Billing\Invoice;

// Detect module name from filename.
$gatewayModuleName = 'pay2s';

// Fetch gateway configuration parameters.
$gatewayParams = getGatewayVariables($gatewayModuleName);

// Define response function
function response($data, $code = 200)
{
    http_response_code($code);
    header('Content-Type: application/json');
    echo json_encode($data);
    die();
}

// Die if module is not active.
if (!$gatewayParams['type']) {
    die("Module Not Activated");
}

// Handle POST request
$rawData = file_get_contents('php://input');
$data = json_decode($rawData, true);
if (is_null($data)) {
    echo 'Invalid JSON data.';
    exit;
}
try {

   // $accessKey = $data['accessKey'];
   // $secretKey = $gatewayParams["secretKey"];
    $amount = $data['amount'];
    $extraData = isset($data['extraData']) ? $data['extraData'] : '';
    $message = $data['message'];
    $orderId = $data['orderId'];
    $orderInfo = $data['orderInfo'];
    $orderType = $data['orderType'];
    $partnerCode = $data['partnerCode'];
    $payType = $data['payType'];
    $requestId = $data['requestId'];
    $responseTime = $data['responseTime'];
    $resultCode = $data['resultCode'];
    $transId = $data['transId'];
    $m2signature = $data['m2signature'];
    
    $accessKey = $gatewayParams["access_key"];
    $secretKey = $gatewayParams["secret_key"];

    // Log IPN data
    

    // Generate signature for validation
    $rawHash = "accessKey=$accessKey&amount=$amount&extraData=$extraData&message=$message&orderId=$orderId&orderInfo=$orderInfo&orderType=$orderType&partnerCode=$partnerCode&payType=$payType&requestId=$requestId&responseTime=$responseTime&resultCode=$resultCode&transId=$transId";
    
    $partnerSignature = hash_hmac("sha256", $rawHash, $secretKey);

    // Validate signature
    if ($m2signature != $partnerSignature) {
        response(['success' => false, 'message' => 'ERROR! Fail checksum'], 200);
    }
    
  //  $logFile = __DIR__ . '/pay2s_ipn.log';
  //  file_put_contents($logFile, date('Y-m-d H:i:s') . " - m2signature: " . print_r($m2signature, true) . PHP_EOL, FILE_APPEND);
   //  file_put_contents($logFile, date('Y-m-d H:i:s') . " - partnerSignature: " . print_r($errorCode, true) . PHP_EOL, FILE_APPEND);

    // Check error code
    if ($resultCode != '0') {
        response(['success' => true, 'message' => 'Error Code: ' . $errorCode], 200);
    }

    // Validate invoice ID
    $invoiceId = checkCbInvoiceID($orderId, $gatewayModuleName);
  //   file_put_contents($logFile, date('Y-m-d H:i:s') . " - invoiceId: " . print_r($invoiceId, true) . PHP_EOL, FILE_APPEND);
    $order = Capsule::table('tblinvoices')->find($invoiceId);
    if (!$order) {
        response(['success' => false, 'message' => 'No existing invoice found!'], 404);
    }

    // Check if invoice already paid
    if ($order->status == 'Paid') {
        response(['success' => true, 'message' => 'Invoice has already been paid!'], 201);
    }

    // Add payment to invoice
    addInvoicePayment($invoiceId, $transId, $amount, 0, $gatewayModuleName);
    logTransaction($gatewayModuleName, $_POST, 'Success');
    response(['success' => true, 'message' => 'Received payment result success'], 200);

} catch (Exception $e) {
    logTransaction($gatewayModuleName, $_POST, 'Error');
    response(['success' => false, 'message' => $e->getMessage()], 500);
}
