<?php

/**
 * WHMCS Sample Payment Callback File
 *
 * This sample file demonstrates how a payment gateway callback should be
 * handled within WHMCS.
 *
 * It demonstrates verifying that the payment gateway module is active,
 * validating an Invoice ID, checking for the existence of a Transaction ID,
 * Logging the Transaction for debugging and Adding Payment to an Invoice.
 *
 * For more information, please refer to the online documentation.
 *
 * @see https://developers.whmcs.com/payment-gateways/callbacks/
 *
 * @copyright Copyright (c) WHMCS Limited 2017
 * @license http://www.whmcs.com/license/ WHMCS Eula
 */

// Require libraries needed for gateway module functions.
require_once __DIR__ . '/../../../init.php';
require_once __DIR__ . '/../../../includes/gatewayfunctions.php';
require_once __DIR__ . '/../../../includes/invoicefunctions.php';

use Illuminate\Database\Capsule\Manager as Capsule;

// Detect module name from filename.
$gatewayModuleName = 'pay2s';


// Fetch gateway configuration parameters.
$gatewayParams = getGatewayVariables($gatewayModuleName);

// Die if module is not active.
if (!$gatewayParams['type']) {
    die("Module Not Activated");
}

// Sanitize GET parameters
$partnerCode = filter_input(INPUT_GET, 'partnerCode', FILTER_SANITIZE_STRING);
$orderId = filter_input(INPUT_GET, 'orderId', FILTER_SANITIZE_STRING);
$message = filter_input(INPUT_GET, 'message', FILTER_SANITIZE_STRING);
$transId = filter_input(INPUT_GET, 'transId', FILTER_SANITIZE_STRING);
$orderInfo = filter_input(INPUT_GET, 'orderInfo', FILTER_SANITIZE_STRING);
$amount = filter_input(INPUT_GET, 'amount', FILTER_SANITIZE_STRING);
$resultCode = filter_input(INPUT_GET, 'resultCode', FILTER_SANITIZE_STRING);
$responseTime = filter_input(INPUT_GET, 'responseTime', FILTER_SANITIZE_STRING);
$requestId = filter_input(INPUT_GET, 'requestId', FILTER_SANITIZE_STRING);
$extraData = filter_input(INPUT_GET, 'extraData', FILTER_SANITIZE_STRING);
$payType = filter_input(INPUT_GET, 'payType', FILTER_SANITIZE_STRING);
$orderType = filter_input(INPUT_GET, 'orderType', FILTER_SANITIZE_STRING);
$m2signature = filter_input(INPUT_GET, 'm2signature', FILTER_SANITIZE_STRING);

// Gateway parameters
$accessKey = $gatewayParams["access_key"];
$secretKey = $gatewayParams["secret_key"];

// Generate hash for signature validation
$rawHash = "accessKey=$accessKey&amount=$amount&message=$message&orderId=$orderId&orderInfo=$orderInfo&orderType=$orderType&partnerCode=$partnerCode&payType=$payType&requestId=$requestId&responseTime=$responseTime&resultCode=$resultCode";
//$rawHash = "accessKey=$accessKey&amount=$amount&message=$message&orderId=$orderId&orderInfo=$orderInfo&orderType=$orderType&partnerCode=$partnerCode&payType=$payType&requestId=$requestId&responseTime=$responseTime&resultCode=$resultCode";
$partnerSignature = hash_hmac("sha256", $rawHash, $secretKey);
//$logFile = __DIR__ . '/pay2s_reponse.log';
//file_put_contents($logFile, date('Y-m-d H:i:s') . " - IPN Data: " . print_r($rawHash, true) . PHP_EOL, FILE_APPEND);
//echo json_encode($partnerSignature);

// Validate signature
$paymentSuccess = false;
if ($m2signature === $partnerSignature) {
    if ($resultCode === '0') {
        try {
            $paymentSuccess = true;
        } catch (Exception $e) {
            // Log the error
            logTransaction($gatewayModuleName, $_GET, 'Duplicate Transaction ID');
        }
    }
}

// Redirect to WHMCS with result
callback3DSecureRedirect($orderId, $paymentSuccess);
