<?php

function pay2s_MetaData()
{
    return array(
        'DisplayName' => 'Cổng thanh toán Pay2S',
        'APIVersion' => '1.1',
        // Use API Version 1.1
        'DisableLocalCredtCardInput' => true,
        'TokenisedStorage' => false,
    );
}

function pay2s_config()
{
    return array(
        // the friendly display name for a payment gateway should be
        // defined here for backwards compatibility
        'FriendlyName' => array(
            'Type' => 'System',
            'Value' => 'Cổng thanh toán Pay2S',
            'Description' => 'Thực hiện Thanh toán qua cổng thanh toán Pay2S',
        ),
        // // a password field type allows for masked text input
        'order_desc' => array(
            'FriendlyName' => 'Nội dung thanh toán',
            'Type' => 'text',
            'Default' => 'DH{{orderid}}',
            'Description' => 'Nội dung thanh toán giúp nhận biết thanh toán cho đơn hàng nào. Sử dụng {{orderid}} để chèn mã đơn hàng.',
        ),
        'bank_accounts' => array(
            'FriendlyName' => 'Bank Accounts',
            'Type' => 'textarea',
            'Rows' => '5',
            'Description' => 'Enter each account in the format: AccountNumber|BankCode',
        ),
        'partner_code' => array(
            'FriendlyName' => 'Partner Code',
            'Type' => 'text',
            'Default' => '',
            'Description' => 'Được cung cấp bởi Pay2S',
        ),
        'access_key' => array(
            'FriendlyName' => 'Access Key',
            'Type' => 'password',
            'Default' => '',
            'Description' => 'Được cung cấp bởi Pay2S.',
        ),
        'secret_key' => array(
            'FriendlyName' => 'Secret Key',
            'Type' => 'password',
            'Default' => '',
            'Description' => 'Được cung cấp bởi Pay2S.',
        ),
        'endpoint' => array(
            'FriendlyName' => 'Url Endpoint',
            'Type' => 'text',
            'Default' => 'https://payment.pay2s.vn/v1/gateway/api/create',
            'Description' => 'https://payment.pay2s.vn/v1/gateway/api/create',
        ),
    );
}
/**
 * Payment link.
 *
 * Required by third party payment gateway modules only.
 *
 * Defines the HTML output displayed on an invoice. Typically consists of an
 * HTML form that will take the user to the payment gateway endpoint.
 *
 * @param array $params Payment Gateway Module Parameters
 *
 * @see https://developers.whmcs.com/payment-gateways/third-party-gateway/
 *
 * @return string
 */
function pay2s_link($params)
{
    // Gateway Configuration Parameters
    $merchantName = $params['companyname'];
    $partnerCode = $params['partner_code'];
    $accessKey = $params['access_key'];
    $secretKey = $params['secret_key'];
    $endpoint = $params['endpoint'];
    $systemUrl = $params['systemurl'];
    $redirectUrl = $systemUrl . 'modules/gateways/callback/pay2s.php';
    $ipnUrl = $systemUrl . 'modules/gateways/callback/ipn_pay2s.php';
    $bank_accounts = $params['bank_accounts'];
    
    $lines = explode("\n", $bank_accounts);
    $bankList = [];
    foreach ($lines as $line) {
    // Tách từng dòng thành account_number và bank_id
    list($accountNumber, $bankId) = explode('|', trim($line));
    // Đưa các giá trị này vào mảng kết quả
    $bankList[] = [
        'account_number' => trim($accountNumber),
        'bank_id' => trim($bankId)
    ];
    }
    // Invoice Parameters
    $requestId = time() . "";
    $orderId = $params['invoiceid'];
    $orderInfo = str_replace('{{orderid}}', $params['invoiceid'], $params['order_desc']);
    $amount = number_format($params['amount'], 0, '.', '');

    $currencyCode = $params['currency'];
    $extraData = "";
    $requestId = time() . "";
    $requestType = 'pay2s';

    // Client Parameters
    $firstname = $params['clientdetails']['firstname'];
    $lastname = $params['clientdetails']['lastname'];
    $email = $params['clientdetails']['email'];
    $address1 = $params['clientdetails']['address1'];
    $address2 = $params['clientdetails']['address2'];
    $city = $params['clientdetails']['city'];
    $state = $params['clientdetails']['state'];
    $postcode = $params['clientdetails']['postcode'];
    $country = $params['clientdetails']['country'];
    $phone = $params['clientdetails']['phonenumber'];

    // System Parameters
    $companyName = $params['companyname'];
    $systemUrl = $params['systemurl'];
    $langPayNow = $params['langpaynow'];
    $moduleDisplayName = $params['name'];
    $moduleName = $params['paymentmethod'];
    $whmcsVersion = $params['whmcsVersion'];
    $rawHash = "accessKey=" . $accessKey . "&amount=" . $amount . "&bankAccounts=" . $bankList . "&ipnUrl=" . $ipnUrl . "&orderId=" . $orderId . "&orderInfo=" . $orderInfo . "&partnerCode=" . $partnerCode . "&redirectUrl=" . $redirectUrl . "&requestId=" . $requestId . "&requestType=" . $requestType;
    $signature = hash_hmac("sha256", $rawHash, $secretKey);
    $data = array(
            'accessKey' => $accessKey,
            'partnerCode' => $partnerCode,
            'partnerName' => $merchantName,
            'requestId' => $requestId,
            'amount' => $amount,
            'orderId' => $orderId,
            'orderInfo' => $orderInfo,
            'orderType' => $requestType,
            'bankAccounts' => $bankList,
            'redirectUrl' => $redirectUrl,
            'ipnUrl' => $ipnUrl,
            'requestType' => $requestType,
            'signature' => $signature,
        );
    $result = execPostRequest($endpoint, json_encode($data));
    $jsonResult = json_decode($result, true);
    $logFile = __DIR__ . '/pay2s_gateway.log';
    file_put_contents($logFile, date('Y-m-d H:i:s') . " - IPN Data: " . print_r($data, true) . PHP_EOL, FILE_APPEND);
    file_put_contents($logFile, date('Y-m-d H:i:s') . " - IPN Data: " . print_r($jsonResult, true) . PHP_EOL, FILE_APPEND);
    $url = $jsonResult['payUrl'];
    $htmlOutput = '<a href="' . $url . '"><input type="submit" value="' . $langPayNow . '" /></a>';
    return $htmlOutput;
}

function execPostRequest($url, $data)
{
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
    curl_setopt(
        $ch,
        CURLOPT_HTTPHEADER,
        array(
            'Content-Type: application/json;charset=UTF-8'
        )
    );
    curl_setopt($ch, CURLOPT_TIMEOUT, 5);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
    $result = curl_exec($ch);
    curl_close($ch);
    return $result;
}
